# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTIBILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
# This addon was created with the Serpens - Visual Scripting Addon.
# This code is generated from nodes and is not intended for manual editing.
# You can find out more about Serpens at <https://blendermarket.com/products/serpens>.


bl_info = {
    "name": "Grab-it",
    "description": "Grabber Addon for Blender",
    "author": "Xane",
    "version": (1, 0, 5),
    "blender": (2, 93, 4),
    "location": "N-Panel",
    "warning": "",
    "wiki_url": "xanegraphics.com",
    "tracker_url": "",
    "category": "Animation"
}


###############   IMPORTS
import bpy
from bpy.utils import previews
import os
import math
from .translation.translate import GetTranslationDict



###############   INITALIZE VARIABLES
###############   SERPENS FUNCTIONS
def exec_line(line):
    exec(line)

def sn_print(tree_name, *args):
    if tree_name in bpy.data.node_groups:
        item = bpy.data.node_groups[tree_name].sn_graphs[0].prints.add()
        for arg in args:
            item.value += str(arg) + ";;;"
        for area in bpy.context.screen.areas:
            area.tag_redraw()
    print(*args)

def sn_cast_string(value):
    return str(value)

def sn_cast_boolean(value):
    if type(value) == tuple:
        for data in value:
            if bool(data):
                return True
        return False
    return bool(value)

def sn_cast_float(value):
    if type(value) == str:
        try:
            value = float(value)
            return value
        except:
            return float(bool(value))
    elif type(value) == tuple:
        return float(value[0])
    elif type(value) == list:
        return float(len(value))
    elif not type(value) in [float, int, bool]:
        try:
            value = len(value)
            return float(value)
        except:
            return float(bool(value))
    return float(value)

def sn_cast_int(value):
    return int(sn_cast_float(value))

def sn_cast_boolean_vector(value, size):
    if type(value) in [str, bool, int, float]:
        return_value = []
        for i in range(size):
            return_value.append(bool(value))
        return tuple(return_value)
    elif type(value) == tuple:
        return_value = []
        for i in range(size):
            return_value.append(bool(value[i]) if len(value) > i else bool(value[0]))
        return tuple(return_value)
    elif type(value) == list:
        return sn_cast_boolean_vector(tuple(value), size)
    else:
        try:
            value = tuple(value)
            return sn_cast_boolean_vector(value, size)
        except:
            return sn_cast_boolean_vector(bool(value), size)

def sn_cast_float_vector(value, size):
    if type(value) in [str, bool, int, float]:
        return_value = []
        for i in range(size):
            return_value.append(sn_cast_float(value))
        return tuple(return_value)
    elif type(value) == tuple:
        return_value = []
        for i in range(size):
            return_value.append(sn_cast_float(value[i]) if len(value) > i else sn_cast_float(value[0]))
        return tuple(return_value)
    elif type(value) == list:
        return sn_cast_float_vector(tuple(value), size)
    else:
        try:
            value = tuple(value)
            return sn_cast_float_vector(value, size)
        except:
            return sn_cast_float_vector(sn_cast_float(value), size)

def sn_cast_int_vector(value, size):
    return tuple(map(int, sn_cast_float_vector(value, size)))

def sn_cast_color(value, use_alpha):
    length = 4 if use_alpha else 3
    value = sn_cast_float_vector(value, length)
    tuple_list = []
    for data in range(length):
        data = value[data] if len(value) > data else value[0]
        tuple_list.append(sn_cast_float(min(1, max(0, data))))
    return tuple(tuple_list)

def sn_cast_list(value):
    if type(value) in [str, tuple, list]:
        return list(value)
    elif type(value) in [int, float, bool]:
        return [value]
    else:
        try:
            value = list(value)
            return value
        except:
            return [value]

def sn_cast_blend_data(value):
    if hasattr(value, "bl_rna"):
        return value
    elif type(value) in [tuple, bool, int, float, list]:
        return None
    elif type(value) == str:
        try:
            value = eval(value)
            return value
        except:
            return None
    else:
        return None

def sn_cast_enum(string, enum_values):
    for item in enum_values:
        if item[1] == string:
            return item[0]
        elif item[0] == string.upper():
            return item[0]
    return string


###############   IMPERATIVE CODE
###############   EVALUATED CODE
#######   Grab-it
class SNA_OT_Drop(bpy.types.Operator):
    bl_idname = "sna.drop"
    bl_label = "Drop"
    bl_description = "Drop object at current frame"
    bl_options = {"REGISTER", "UNDO"}


    @classmethod
    def poll(cls, context):
        return True

    def execute(self, context):
        try:
            pass # Drop Script Start
            import bpy
            if(bpy.context.mode != 'OBJECT'):
                bpy.ops.object.mode_set(mode = 'OBJECT')
            bpy.ops.object.select_all(action='DESELECT')
            #bpy.data.objects["Grabber_Handle"].select_set(True)
            #bpy.context.view_layer.objects.active = bpy.data.objects['Grabber_Handle']
            #bpy.ops.object.modifier_apply_as_shapekey(keep_modifier=True, modifier="Hook-Target_Empty")
            #bpy.context.object.modifiers["Hook-Target_Empty"].show_viewport = False
            #bpy.context.object.modifiers["Hook-Target_Empty"].show_render = False
            #bpy.context.object.modifiers["Hook-Target_Empty"].keyframe_insert("show_viewport")
            #bpy.context.object.modifiers["Hook-Target_Empty"].keyframe_insert("show_render")
            bpy.context.scene.frame_current -= 1
            bpy.data.objects["Grabber_Empty"].select_set(True)
            bpy.context.view_layer.objects.active = bpy.data.objects['Grabber_Empty']
            bpy.context.area.type = 'VIEW_3D'
            bpy.ops.view3d.snap_cursor_to_active()
            bpy.ops.object.select_all(action='DESELECT')
            bpy.data.objects["Grabber_Handle"].select_set(True)
            bpy.context.view_layer.objects.active = bpy.data.objects['Grabber_Handle']
            bpy.context.object.hide_viewport = False
            bpy.data.objects["Grabber_Handle"].select_set(True)
            bpy.context.view_layer.objects.active = bpy.data.objects['Grabber_Handle']
            bpy.ops.view3d.snap_selected_to_cursor(use_offset=False)
            bpy.ops.anim.keyframe_insert_menu(type='Location')
            bpy.context.object.hide_viewport = True
            bpy.context.object.modifiers["Hook-Target_Empty"].keyframe_insert("strength")
            bpy.context.object.constraints["Copy Rotation"].keyframe_insert(data_path="influence")
            bpy.context.scene.frame_current += 1
            bpy.context.object.constraints["Copy Rotation"].influence = 0
            bpy.context.object.constraints["Copy Rotation"].keyframe_insert(data_path="influence")
            bpy.ops.object.modifier_apply_as_shapekey(keep_modifier=True, modifier="Hook-Target_Empty")
            bpy.context.scene.frame_current -= 1
            for shapekey in bpy.data.shape_keys:
                for i, keyblock in enumerate(shapekey.key_blocks):
                    if keyblock.name == "Hook-Target_Empty":
                            keyblock.value = 0
                            keyblock.keyframe_insert("value")
            bpy.context.object.constraints["Copy Rotation"].influence = 1
            bpy.context.object.constraints["Copy Rotation"].keyframe_insert(data_path="influence")
            bpy.context.scene.frame_current += 1
            for shapekey in bpy.data.shape_keys:
                for i, keyblock in enumerate(shapekey.key_blocks):
                    if keyblock.name == "Hook-Target_Empty":
                            keyblock.value = 1
                            keyblock.keyframe_insert("value")
            bpy.context.object.modifiers["Hook-Target_Empty"].strength = 0
            bpy.context.object.modifiers["Hook-Target_Empty"].keyframe_insert("strength")
            bpy.context.object.constraints["Copy Rotation"].influence = 0
            bpy.context.object.constraints["Copy Rotation"].keyframe_insert(data_path="influence")
            bpy.data.scenes[bpy.context.scene.name].timeline_markers.new('Drop', frame=bpy.context.scene.frame_current)
            for shapekey in bpy.data.shape_keys:
                for i, keyblock in enumerate(shapekey.key_blocks):
                    if keyblock.name == "Hook-Target_Empty":
                            keyblock.name = "Hook-Target_Empty_Used"
            bpy.ops.object.select_all(action='DESELECT')
            bpy.data.objects["Grabber_Handle"].select_set(True)
            bpy.context.view_layer.objects.active = bpy.data.objects['Grabber_Handle']
            bpy.context.object.name = "Used_Grabber_Handle"
            bpy.ops.object.select_all(action='DESELECT')
            bpy.data.objects["Grabber_Empty"].select_set(True)
            bpy.context.view_layer.objects.active = bpy.data.objects['Grabber_Empty']
            bpy.context.object.name = "Used_Grabber_Empty"
            bpy.ops.object.select_all(action='DESELECT')
            bpy.data.objects["Target_Empty"].select_set(True)
            bpy.context.view_layer.objects.active = bpy.data.objects['Target_Empty']
            bpy.context.object.name = "Used_Target_Empty"
            bpy.ops.object.select_all(action='DESELECT')
            pass # Drop Script End
        except Exception as exc:
            print(str(exc) + " | Error in execute function of Drop")
        return {"FINISHED"}

    def invoke(self, context, event):
        try:
            pass
        except Exception as exc:
            print(str(exc) + " | Error in invoke function of Drop")
        return self.execute(context)


class SNA_OT_Parent(bpy.types.Operator):
    bl_idname = "sna.parent"
    bl_label = "Parent"
    bl_description = "Parent active object(s) at this frame"
    bl_options = {"REGISTER", "UNDO"}


    @classmethod
    def poll(cls, context):
        return True

    def execute(self, context):
        try:
            pass # Parent Script Start
            import bpy

            def checkParent():
                if(bpy.data.objects[bpy.context.active_object.name].parent):
                    if (bpy.data.objects[bpy.context.active_object.name].parent.name.startswith('Used_Grabber_Empty')):
                        Father = bpy.data.objects[bpy.context.active_object.name].parent
                        GrandFather = Father.parent
                        if (bpy.context.active_object.name.startswith('Used_Grabber_Handle')):
                            bpy.context.object.hide_viewport = True
                        bpy.ops.object.select_all(action='DESELECT')
                        bpy.data.objects[GrandFather.name].select_set(True)
                        bpy.context.view_layer.objects.active = bpy.data.objects[GrandFather.name]
                        bpy.context.object.hide_viewport = False
                        bpy.data.objects[GrandFather.name].select_set(True)
                        bpy.context.view_layer.objects.active = bpy.data.objects[GrandFather.name]
                        checkParent()
                    else:
                        return
            checkParent()
            self = bpy.data.objects[bpy.context.active_object.name]
            if ("Grabber_Empty" in  bpy.data.objects):
                bpy.data.objects["Grabber_Empty"].select_set(True)
                bpy.context.view_layer.objects.active = bpy.data.objects['Grabber_Empty']
                bpy.ops.object.parent_set(type='OBJECT', keep_transform=True)
                bpy.ops.object.select_all(action='DESELECT')
            else:
                bpy.data.objects["Used_Grabber_Empty"].select_set(True)
                bpy.context.view_layer.objects.active = bpy.data.objects['Used_Grabber_Empty']
                bpy.ops.object.parent_set(type='OBJECT', keep_transform=True)
                bpy.ops.object.select_all(action='DESELECT')
            bpy.ops.object.select_all(action='DESELECT')
            bpy.data.objects[self.name].select_set(True)
            bpy.context.view_layer.objects.active = bpy.data.objects[self.name]
            if(self.name.startswith('Used_Grabber_Handle')):
                bpy.context.object.hide_viewport = True
            bpy.ops.object.select_all(action='DESELECT')
            pass # Parent Script End
        except Exception as exc:
            print(str(exc) + " | Error in execute function of Parent")
        return {"FINISHED"}

    def invoke(self, context, event):
        try:
            pass
        except Exception as exc:
            print(str(exc) + " | Error in invoke function of Parent")
        return self.execute(context)


class SNA_OT_Clear(bpy.types.Operator):
    bl_idname = "sna.clear"
    bl_label = "Clear"
    bl_description = "Delete All Grabber Setups"
    bl_options = {"REGISTER", "UNDO"}


    @classmethod
    def poll(cls, context):
        return True

    def execute(self, context):
        try:
            pass # Clear Script Start
            import bpy
            collection = bpy.data.collections.get('Grabber')
            for obj in collection.objects:
                bpy.data.objects.remove(obj, do_unlink=True)
            bpy.data.collections.remove(collection)
            while bpy.data.scenes[bpy.context.scene.name].timeline_markers.get('Grab'):
                grabMarker = bpy.data.scenes[bpy.context.scene.name].timeline_markers.get('Grab')
                bpy.data.scenes[bpy.context.scene.name].timeline_markers.remove(grabMarker)
            while bpy.data.scenes[bpy.context.scene.name].timeline_markers.get('Drop'):
                dropMarker = bpy.data.scenes[bpy.context.scene.name].timeline_markers.get('Drop')
                bpy.data.scenes[bpy.context.scene.name].timeline_markers.remove(dropMarker)
            pass # Clear Script End
        except Exception as exc:
            print(str(exc) + " | Error in execute function of Clear")
        return {"FINISHED"}

    def invoke(self, context, event):
        try:
            pass
        except Exception as exc:
            print(str(exc) + " | Error in invoke function of Clear")
        return context.window_manager.invoke_confirm(self, event)


class SNA_OT_Clear_Markers(bpy.types.Operator):
    bl_idname = "sna.clear_markers"
    bl_label = "Clear Markers"
    bl_description = "Delete All Grabber Markers"
    bl_options = {"REGISTER", "UNDO"}


    @classmethod
    def poll(cls, context):
        return True

    def execute(self, context):
        try:
            pass # ClearMarkers Script Start
            import bpy
            while bpy.data.scenes[bpy.context.scene.name].timeline_markers.get('Grab'):
                grabMarker = bpy.data.scenes[bpy.context.scene.name].timeline_markers.get('Grab')
                bpy.data.scenes[bpy.context.scene.name].timeline_markers.remove(grabMarker)
            while bpy.data.scenes[bpy.context.scene.name].timeline_markers.get('Drop'):
                dropMarker = bpy.data.scenes[bpy.context.scene.name].timeline_markers.get('Drop')
                bpy.data.scenes[bpy.context.scene.name].timeline_markers.remove(dropMarker)
            pass # ClearMarkers Script End
        except Exception as exc:
            print(str(exc) + " | Error in execute function of Clear Markers")
        return {"FINISHED"}

    def invoke(self, context, event):
        try:
            pass
        except Exception as exc:
            print(str(exc) + " | Error in invoke function of Clear Markers")
        return self.execute(context)


class SNA_PT_Clear_F89BE(bpy.types.Panel):
    bl_label = "Clear"
    bl_idname = "SNA_PT_Clear_F89BE"
    bl_space_type = "VIEW_3D"
    bl_region_type = "WINDOW"
    bl_order = 0


    @classmethod
    def poll(cls, context):
        return True

    def draw_header(self, context):
        try:
            layout = self.layout
        except Exception as exc:
            print(str(exc) + " | Error in Clear panel header")

    def draw(self, context):
        try:
            layout = self.layout
            op = layout.operator("sna.clear",text=r"Clear All",emboss=True,depress=False,icon_value=3)
            op = layout.operator("sna.clear_markers",text=r"Markers",emboss=True,depress=False,icon_value=509)
        except Exception as exc:
            print(str(exc) + " | Error in Clear panel")


class SNA_OT_Grab(bpy.types.Operator):
    bl_idname = "sna.grab"
    bl_label = "Grab"
    bl_description = ""
    bl_options = {"REGISTER", "UNDO"}


    @classmethod
    def poll(cls, context):
        return True

    def execute(self, context):
        try:
            pass # Grab Script Start
            import bpy
            #if(bpy.data.objects["Target_Empty"].type == 'ARMATURE'):
            #    saveName = bpy.context.active_object.name
            if ("Grabber_Empty" in  bpy.data.objects):
                self.report({'ERROR'}, "You haven't dropped what you grabbed yet (O.O)")
            else:
                bpy.context.object.name = "Target_Empty"
                bpy.context.area.type = 'VIEW_3D'
                if(bpy.data.objects["Target_Empty"].type == 'ARMATURE'):
                    if(bpy.context.object.mode != 'POSE'):
                        bpy.ops.object.posemode_toggle()
                        bpy.ops.view3d.snap_cursor_to_active()
                        bpy.ops.object.posemode_toggle()
                    else:
                        bpy.ops.view3d.snap_cursor_to_active()
                        bpy.ops.object.posemode_toggle()
                else:
                    bpy.ops.view3d.snap_cursor_to_selected()
                bpy.context.view_layer.active_layer_collection = bpy.context.view_layer.layer_collection
                #bpy.context.area.type = 'TEXT_EDITOR'
                bpy.ops.mesh.primitive_cube_add(enter_editmode=False, align='WORLD', location=(bpy.context.scene.cursor.location), scale=(0.1, 0.1, 0.1))
                if ('Grabber' in bpy.data.collections):
                    bpy.data.collections['Grabber'].objects.link(bpy.context.active_object)
                    bpy.context.scene.collection.objects.unlink(bpy.context.active_object)
                else:
                    bpy.ops.object.move_to_collection(collection_index=0, is_new=True, new_collection_name="Grabber")
                    #bpy.data.collections['Grabber'].objects.link(bpy.context.active_object)
                    #bpy.context.scene.collection.objects.unlink(bpy.context.active_object)
                bpy.ops.object.transform_apply(location=False, rotation=False, scale=True)
                bpy.context.object.name = "Grabber_Handle"
                bpy.ops.object.constraint_add(type='COPY_ROTATION')
                bpy.context.object.constraints["Copy Rotation"].influence = 0
                bpy.context.object.constraints["Copy Rotation"].target = bpy.data.objects["Target_Empty"]
                bpy.context.object.constraints["Copy Rotation"].keyframe_insert(data_path="influence")
                bpy.context.scene.frame_current -= 1
                bpy.context.object.constraints["Copy Rotation"].influence = 0
                bpy.context.object.constraints["Copy Rotation"].keyframe_insert(data_path="influence")
                bpy.context.scene.frame_current += 1
                bpy.ops.anim.keyframe_insert_menu(type='Location')
                bpy.ops.object.editmode_toggle()
                bpy.ops.mesh.select_all(action='DESELECT')
                bpy.ops.mesh.select_all(action='SELECT')
                bpy.ops.object.editmode_toggle()
                bpy.ops.object.empty_add(type='SPHERE', align='WORLD', location=(bpy.context.scene.cursor.location), scale=(1, 1, 1))
                bpy.context.object.name = "Grabber_Empty"
                bpy.ops.object.select_all(action='DESELECT')
                bpy.data.objects["Grabber_Empty"].select_set(True)
                bpy.context.view_layer.objects.active = bpy.data.objects['Grabber_Empty']
                if ('Grabber' in bpy.data.collections):
                    FoundGrabber = False
                    for col in bpy.context.scene.collection.children:
                        if col.name == 'Grabber':
                            bpy.data.collections['Grabber'].objects.link(bpy.context.active_object)
                            bpy.context.scene.collection.objects.unlink(bpy.context.active_object)
                            FoundGrabber = True
                            #Do Nothing
                    if(FoundGrabber == False):
                        myCol = bpy.data.collections["Grabber"]
                        bpy.context.scene.collection.children.link(myCol)
                        bpy.data.collections['Grabber'].objects.link(bpy.context.active_object)
                        bpy.context.scene.collection.objects.unlink(bpy.context.active_object)
                else:
                    bpy.ops.object.move_to_collection(collection_index=0, is_new=True, new_collection_name="Grabber")
                bpy.context.object.empty_display_size = 0.0
                bpy.data.objects["Grabber_Empty"].select_set(True)
                bpy.data.objects["Grabber_Handle"].select_set(True)
                bpy.context.view_layer.objects.active = bpy.data.objects['Grabber_Handle']
                bpy.ops.object.parent_set(type='VERTEX_TRI')
                bpy.ops.object.select_all(action='DESELECT')
                bpy.data.objects["Target_Empty"].select_set(True)
                bpy.data.objects["Grabber_Handle"].select_set(True)
                bpy.context.view_layer.objects.active = bpy.data.objects['Grabber_Handle']
                bpy.ops.object.editmode_toggle()
                bpy.ops.mesh.select_all(action='DESELECT')
                bpy.ops.mesh.select_all(action='SELECT')
                if(bpy.data.objects["Target_Empty"].type == 'ARMATURE'):
                    bpy.ops.object.hook_add_selob(use_bone=True)
                else:
                    bpy.ops.object.hook_add_selob(use_bone=False)
                bpy.ops.object.editmode_toggle()
                bpy.context.scene.frame_current += 1
                bpy.context.object.modifiers["Hook-Target_Empty"].keyframe_insert("show_viewport")
                bpy.context.object.modifiers["Hook-Target_Empty"].keyframe_insert("show_render")
                bpy.context.object.modifiers["Hook-Target_Empty"].keyframe_insert("strength")
                bpy.context.scene.frame_current -= 1
                bpy.context.object.modifiers["Hook-Target_Empty"].keyframe_insert("show_viewport")
                bpy.context.object.modifiers["Hook-Target_Empty"].keyframe_insert("show_render")
                bpy.context.object.modifiers["Hook-Target_Empty"].strength = 0
                bpy.context.object.modifiers["Hook-Target_Empty"].keyframe_insert("strength")
                bpy.context.scene.frame_current -= 1
                bpy.context.object.modifiers["Hook-Target_Empty"].show_viewport = True
                bpy.context.object.modifiers["Hook-Target_Empty"].show_render = True
                bpy.context.object.modifiers["Hook-Target_Empty"].keyframe_insert("show_viewport")
                bpy.context.object.modifiers["Hook-Target_Empty"].keyframe_insert("show_render")
                bpy.context.object.modifiers["Hook-Target_Empty"].strength = 0
                bpy.context.object.modifiers["Hook-Target_Empty"].keyframe_insert("strength")
                bpy.context.scene.frame_current += 1
                bpy.data.scenes[bpy.context.scene.name].timeline_markers.new('Grab', frame=bpy.context.scene.frame_current)
                bpy.data.objects["Grabber_Handle"].select_set(True)
                bpy.context.view_layer.objects.active = bpy.data.objects['Grabber_Handle']
                bpy.context.object.hide_viewport = True
                bpy.context.object.hide_render = True
                bpy.ops.object.select_all(action='DESELECT')
            pass # Grab Script End
        except Exception as exc:
            print(str(exc) + " | Error in execute function of Grab")
        return {"FINISHED"}

    def invoke(self, context, event):
        try:
            pass
        except Exception as exc:
            print(str(exc) + " | Error in invoke function of Grab")
        return self.execute(context)


class SNA_PT_GrabIt_7D924(bpy.types.Panel):
    bl_label = "Grab-It"
    bl_idname = "SNA_PT_GrabIt_7D924"
    bl_space_type = "VIEW_3D"
    bl_region_type = "UI"
    bl_category = 'Grab'
    bl_order = 0


    @classmethod
    def poll(cls, context):
        return True

    def draw_header(self, context):
        try:
            layout = self.layout
        except Exception as exc:
            print(str(exc) + " | Error in Grab-It panel header")

    def draw(self, context):
        try:
            layout = self.layout
            layout.label(text=r"Grab-It",icon_value=0)
            op = layout.operator("sna.grab",text=r"Grab",emboss=True,depress=False,icon_value=778)
            op = layout.operator("sna.parent",text=r"Parent",emboss=True,depress=False,icon_value=56)
            op = layout.operator("sna.drop",text=r"Drop",emboss=True,depress=False,icon_value=703)
            layout.popover("SNA_PT_Clear_F89BE",text=r"Clear",icon_value=3)
        except Exception as exc:
            print(str(exc) + " | Error in Grab-It panel")

def sn_append_menu_0C4C2(self,context):
    try:
        layout = self.layout
        op = layout.operator("sna.setstartframe",text=r"Start",emboss=True,depress=False,icon_value=0)
        op = layout.operator("sna.setendframe",text=r"End",emboss=True,depress=False,icon_value=0)
    except Exception as exc:
        print(str(exc) + " | Error in Dopesheet Ht Header when adding to menu")


class SNA_OT_Setstartframe(bpy.types.Operator):
    bl_idname = "sna.setstartframe"
    bl_label = "SetStartFrame"
    bl_description = ""
    bl_options = {"REGISTER", "UNDO"}


    @classmethod
    def poll(cls, context):
        return True

    def execute(self, context):
        try:
            pass # Set Start Frame Script Start
            import bpy
            bpy.data.scenes["Scene"].frame_start = bpy.data.scenes["Scene"].frame_current
            pass # Set Start Frame Script End
        except Exception as exc:
            print(str(exc) + " | Error in execute function of SetStartFrame")
        return {"FINISHED"}

    def invoke(self, context, event):
        try:
            pass
        except Exception as exc:
            print(str(exc) + " | Error in invoke function of SetStartFrame")
        return self.execute(context)


class SNA_OT_Setendframe(bpy.types.Operator):
    bl_idname = "sna.setendframe"
    bl_label = "SetEndFrame"
    bl_description = ""
    bl_options = {"REGISTER", "UNDO"}


    @classmethod
    def poll(cls, context):
        return True

    def execute(self, context):
        try:
            pass # Set End Frame Script Start
            import bpy
            bpy.data.scenes["Scene"].frame_end = bpy.data.scenes["Scene"].frame_current
            pass # Set End Frame Script End
        except Exception as exc:
            print(str(exc) + " | Error in execute function of SetEndFrame")
        return {"FINISHED"}

    def invoke(self, context, event):
        try:
            pass
        except Exception as exc:
            print(str(exc) + " | Error in invoke function of SetEndFrame")
        return self.execute(context)


###############   REGISTER ICONS
def sn_register_icons():
    icons = []
    bpy.types.Scene.grabit_icons = bpy.utils.previews.new()
    icons_dir = os.path.join( os.path.dirname( __file__ ), "icons" )
    for icon in icons:
        bpy.types.Scene.grabit_icons.load( icon, os.path.join( icons_dir, icon + ".png" ), 'IMAGE' )

def sn_unregister_icons():
    bpy.utils.previews.remove( bpy.types.Scene.grabit_icons )


###############   REGISTER PROPERTIES
def sn_register_properties():
    pass

def sn_unregister_properties():
    pass


###############   REGISTER ADDON
def register():
    sn_register_icons()
    sn_register_properties()
    bpy.utils.register_class(SNA_OT_Drop)
    bpy.utils.register_class(SNA_OT_Parent)
    bpy.utils.register_class(SNA_OT_Clear)
    bpy.utils.register_class(SNA_OT_Clear_Markers)
    bpy.utils.register_class(SNA_PT_Clear_F89BE)
    bpy.utils.register_class(SNA_OT_Grab)
    bpy.utils.register_class(SNA_PT_GrabIt_7D924)
    bpy.utils.register_class(SNA_OT_Setstartframe)
    bpy.utils.register_class(SNA_OT_Setendframe)
    bpy.types.DOPESHEET_HT_header.append(sn_append_menu_0C4C2)
    try:
        bpy.app.translations.register(__package__, GetTranslationDict())
    except:
        pass


###############   UNREGISTER ADDON
def unregister():
    sn_unregister_icons()
    sn_unregister_properties()
    bpy.types.DOPESHEET_HT_header.remove(sn_append_menu_0C4C2)
    bpy.utils.unregister_class(SNA_OT_Setendframe)
    bpy.utils.unregister_class(SNA_OT_Setstartframe)
    bpy.utils.unregister_class(SNA_PT_GrabIt_7D924)
    bpy.utils.unregister_class(SNA_OT_Grab)
    bpy.utils.unregister_class(SNA_PT_Clear_F89BE)
    bpy.utils.unregister_class(SNA_OT_Clear_Markers)
    bpy.utils.unregister_class(SNA_OT_Clear)
    bpy.utils.unregister_class(SNA_OT_Parent)
    bpy.utils.unregister_class(SNA_OT_Drop)
    try:
        bpy.app.translations.unregister(__package__)
    except:
        pass